<?php

/**
  Plugin Name: ARForms - Signature Addon for ARForms
  Description: Extension for Arforms to accept signature from user. ( This plugin will work with ARForms plugin only. )
  Version: 2.0
  Plugin URI: http://www.arformsplugin.com/
  Author: Repute InfoSystems
  Author URI: http://reputeinfosystems.com/
  Text Domain: ARForms-Digital-Signature
 */
if ( ! defined( 'ARF_DIG_SIG_DIR' ) ) {
	define( 'ARF_DIG_SIG_DIR', WP_PLUGIN_DIR . '/arformsignature' );
}

global $arfsiteurl;
$arfsiteurl = home_url();
if ( is_ssl() && ( ! preg_match( '/^https:\/\/.*\..*$/', $arfsiteurl ) || ! preg_match( '/^https:\/\/.*\..*$/', WP_PLUGIN_URL ) ) ) {
	$arfsiteurl = str_replace( 'http://', 'https://', $arfsiteurl );
	define( 'ARF_DIG_SIG_URL', str_replace( 'http://', 'https://', WP_PLUGIN_URL . '/arformsignature' ) );
} else {
	define( 'ARF_DIG_SIG_URL', WP_PLUGIN_URL . '/arformsignature' );
}

if ( ! defined( 'ARF_DS_PATH' ) ) {
	define( 'ARF_DS_PATH', WP_PLUGIN_DIR . '/arformsignature' );
}

if ( ! defined( 'ARF_DS_CORE' ) ) {
	define( 'ARF_DS_CORE', ARF_DS_PATH . '/core' );
}

if ( ! defined( 'ARF_SIGNATURE_SLUG' ) ) {
	define( 'ARF_SIGNATURE_SLUG', 'signature' );
}

load_plugin_textdomain( 'ARForms-Digital-Signature', false, 'arformsignature/languages/' );

global $arf_digital_signature, $arf_field_class_name, $arf_new_field_data, $arfajaxurl, $arf_field_image_path, $arfjsonimage, $arfds_memory_limit, $memory_limit;
$arf_digital_signature = new ARF_Digital_Signature();
$arfds_memory_limit    = 256;
$memory_limit          = ini_get( 'memory_limit' );
if ( isset( $memory_limit ) ) {
	$memory_limit = str_replace( 'M', '', $memory_limit );
}

global $arf_dig_sig_version;
$arf_dig_sig_version = '2.0';

global $arf_dig_sig_asset_version;
$arf_dig_sig_asset_version = $arf_dig_sig_version . '.' . rand( 10, 100 );

require_once ARF_DS_CORE . '/signature_to_image_class.php';
$arfjsonimage = new arf_json_image();

global $arformsignatureshortname;
$arformsignatureshortname = 'ARFDIGSIG';

global $arf_signture_loaded;
$arf_signture_loaded = array();

$arfajaxurl           = admin_url( 'admin-ajax.php' );
$arf_field_image_path = array( ARF_SIGNATURE_SLUG => ARF_DIG_SIG_URL . '/images/signature-icon.png' );
$arf_field_class_name = array( ARF_SIGNATURE_SLUG => 'purple' );
$arf_new_field_data   = array( ARF_SIGNATURE_SLUG => esc_html__( 'Signature', 'ARForms-Digital-Signature' ) );

class ARF_Digital_Signature {

	function __construct() {
		global $is_version_compatible;
		$is_version_compatible = $this->is_arforms_version();

		register_activation_hook( __FILE__, array( 'ARF_Digital_Signature', 'install' ) );

		register_activation_hook( __FILE__, array( 'ARF_Digital_Signature', 'arf_signature_check_network_activation' ) );

		register_uninstall_hook( __FILE__, array( 'ARF_Digital_Signature', 'uninstall' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'arf_set_js' ), 11 );

		add_action( 'include_outside_js_css_for_preview_header', array( $this, 'arf_set_js_outside_preview_header' ), 11 );

		add_action( 'admin_enqueue_scripts', array( $this, 'arf_set_css' ), 11 );

		add_filter( 'arf_all_field_css_class_for_editor', array( $this, 'arf_get_field_class' ), 10, 3 );

		add_filter( 'arfavailablefieldsbasicoptions', array( $this, 'add_availablefieldsbasicoptions' ), 10, 3 );

		add_action( 'arfdisplayaddedfields', array( $this, 'add_field_to_editor' ), 11 );

		if ( version_compare( $this->get_arforms_version(), '4.0', '>=' ) ) {
			add_filter( 'arfaavailablefields', array( $this, 'arf_add_signature_field_in_list') );

			add_filter( 'arf_manage_field_element_order_outside', array( $this, 'arf_add_signature_field_in_order') );
		
			add_filter( 'arform_input_fields', array( $this, 'arf_add_signature_field_in_order') );
		}

		if ( $is_version_compatible ) {
			add_action( 'add_label_materialize_outside', array( $this, 'add_label_materialize_outside_editor' ), 11 );
			add_filter( 'arf_input_style_label_position_outside', array( $this, 'add_field_to_array' ), 11, 3 );
			add_filter( 'form_fields', array( $this, 'add_field_to_frontend_v3' ), 11, 10 );
			add_action( 'arfafterbasicfieldlisting', array( $this, 'arf_add_signature_filed_v3' ), 10, 2 );
			add_action( 'wp_enqueue_scripts', array( $this, 'arf_front_footer_js' ), 11, 1 );
			add_action( 'arf_outsite_print_style', array( $this, 'arf_dig_sig_field_css_v3' ), 10, 3 );
		} else {
			add_filter( 'form_fields', array( $this, 'add_field_to_frontend' ), 11, 2 );
			add_action( 'arfafterbasicfieldlisting', array( $this, 'arf_add_signature_filed' ), 10, 2 );
			add_action( 'wp_footer', array( $this, 'arf_front_footer_js' ), 11 );
			add_action( 'arf_outsite_print_style', array( $this, 'arf_dig_sig_field_css' ), 10, 3 );
		}

		add_filter( 'arfdisplayvalue', array( $this, 'arf_display_signature_value' ), 10, 3 );

		add_filter( 'arfpdfdisplayvalue', array( $this, 'arf_display_signature_value_in_pdf' ), 11, 3 );

		add_filter( 'arf_before_create_formentry', array( $this, 'arf_before_create_entry' ), 9, 1 );

		add_filter( 'arf_change_json_default_data_ouside', array( $this, 'arf_new_field_json_filter_outside' ), 10, 1 );

		add_filter( 'arf_new_field_array_filter_outside', array( $this, 'arf_new_field_array_filter_outside' ), 10, 4 );

		add_filter( 'arf_new_field_array_materialize_filter_outside', array( $this, 'arf_new_field_array_filter_outside_materialize' ), 10, 4 );

		add_filter( 'arf_before_destroy_entry', array( $this, 'arf_delete_signature_image' ), 10, 2 );

		add_filter( 'arf_before_createfield', array( $this, 'arf_createfield' ), 10, 2 );

		add_filter( 'arf_brfore_send_mail_chnage_value', array( $this, 'arf_display_signature_value_in_mail' ), 10, 3 );

		add_action( 'arf_outsite_print_style', array( $this, 'arf_create_css' ), 10, 3 );

		add_action( 'admin_notices', array( $this, 'arf_dig_sig_admin_notices' ) );

		add_filter( 'upgrader_pre_install', array( $this, 'arfsignature_backup' ), 10, 2 );

		add_action( 'admin_init', array( $this, 'upgrade_data' ) );

		add_action( 'arf_add_css_from_outside', array( $this, 'arf_add_signature_style' ), 10, 3 );

		add_filter( 'arf_installed_fields_outside', array( $this, 'arf_installed_fields_outside' ), 10 );

		add_filter( 'arf_positioned_field_options_icon', array( $this, 'arf_positioned_field_options_icon_signature' ), 10, 2 );

		add_action( 'wp_ajax_arf_forms_signaturefile_remove', array( $this, 'arf_forms_signaturefile_remove' ) );
	}

	function arf_add_signature_field_in_list( $fields ){
		
		$fields[ARF_SIGNATURE_SLUG] = array(
			'icon' => '<svg viewBox="0 0 30 30"><g id="arfdigitaldownload"><path fill="#4E5462" d="M20.178,11.206l4.407,0.879l-1.989,9.979c-0.186,0.938-1.594,3.99-2.73,4.749l-0.17,0.857c-0.043,0.217-0.218,0.376-0.424,0.408c-0.059,0.01-0.119,0.008-0.181-0.003c-0.278-0.057-0.459-0.326-0.403-0.606l0.16-0.81c-0.81-1.111-0.811-4.691-0.654-5.476L20.178,11.206z M24.647,0.277c-1.211-0.24-2.417,0.837-2.637,1.944l-1.441,7.232l4.407,0.876l1.437-7.23C26.595,2.194,25.862,0.519,24.647,0.277z M18.051,11.089c0.124,0.024,0.247,0.026,0.363,0.009c0.413-0.064,0.762-0.379,0.85-0.813l1.531-7.559c0.113-0.557-0.248-1.1-0.804-1.212c-0.562-0.109-1.101,0.247-1.214,0.805l-1.531,7.559C17.133,10.434,17.495,10.977,18.051,11.089z M16.446,25.152c-0.313-0.217-0.705-0.485-1.295-0.645c-0.485-0.13-0.93-0.071-1.254-0.028c-0.033,0.006-0.072,0.011-0.111,0.015c-0.289-0.521-0.753-0.805-1.39-0.845c-0.841-0.046-1.431,0.44-1.835,0.807c0.004-0.232,0.028-0.513,0.047-0.716c0.057-0.651,0.115-1.324-0.098-1.914c-0.17-0.465-0.631-0.758-1.125-0.717c-1.271,0.127-2.193,0.966-3.005,1.721c-0.853,0.793-1.403,1.272-2.044,1.149c-0.783-0.147-1.172-2.144-1.096-3.987c0.067-1.661,0.192-4.753,1.5-6.353c0.44-0.539,1.105-1.151,1.752-1.006c0.761,0.17,1.538,1.257,1.891,2.643c0.147,0.585,0.734,0.939,1.329,0.791c0.587-0.147,0.94-0.744,0.791-1.33c-0.573-2.258-1.928-3.882-3.535-4.241c-0.963-0.21-2.419-0.08-3.923,1.758c-1.635,2.001-1.887,4.998-1.993,7.649c-0.054,1.324-0.009,5.684,2.876,6.229c1.758,0.333,2.967-0.795,3.939-1.697c0.178-0.166,0.353-0.328,0.524-0.476c-0.058,0.813-0.039,1.668,0.545,2.302c0.872,0.95,1.888,0.91,3.019-0.117c0.044-0.039,0.095-0.084,0.142-0.13c0.155,0.227,0.403,0.463,0.808,0.589c0.489,0.151,0.945,0.091,1.277,0.048c0.142-0.019,0.336-0.044,0.398-0.028c0.217,0.058,0.354,0.15,0.623,0.333c0.201,0.139,0.412,0.281,0.674,0.423c0.218,0.115,0.456,0.149,0.681,0.113	c0.329-0.051,0.632-0.252,0.798-0.568c0.283-0.533,0.08-1.195-0.453-1.479C16.726,25.348,16.584,25.247,16.446,25.152z"/></g>',
			'label' => addslashes( esc_html__('Signature','ARForms-Digital-Signature') )
		);

		return $fields;
	}

	function arf_add_signature_field_in_order( $ordered_list ){

		array_push( $ordered_list, ARF_SIGNATURE_SLUG );

		return $ordered_list;
	}

	function arf_forms_signaturefile_remove() {
		global $wpdb, $arffield, $MdlDb, $armainhelper, $arfieldhelper;

		if ( isset( $_POST['entry_id'] ) && isset( $_POST['field_id'] ) && isset( $_POST['file_url'] ) ) {
			echo $wpdb->query( $wpdb->prepare( 'DELETE FROM ' . $MdlDb->entry_metas . ' WHERE field_id=%d and entry_id= %d', esc_html( $_POST['field_id'] ), esc_html( $_POST['entry_id'] ) ) );
			$image_url = $_POST['file_url'];
			if ( $image_url ) {
				$image_url = ABSPATH . str_replace( site_url() . '/', '', $image_url );
				unlink( $image_url );
			}
		}
		exit;
	}
	function arf_positioned_field_options_icon_signature( $positioned_field_icons, $field_icons ) {
		$positioned_field_icons[ ARF_SIGNATURE_SLUG ] = "{$field_icons['field_require_icon']}" . str_replace( '{arf_field_type}', ARF_SIGNATURE_SLUG, $field_icons['arf_field_duplicate_icon'] ) . "{$field_icons['field_delete_icon']}" . str_replace( '{arf_field_type}', ARF_SIGNATURE_SLUG, $field_icons['field_option_icon'] ) . "{$field_icons['arf_field_move_icon']}";
		return $positioned_field_icons;
	}

	function arf_installed_fields_outside( $fields ) {
		array_push( $fields, 'signature' );
		return $fields;
	}
	function add_field_to_array( $arf_input_style_label_position, $arfinputstyle, $type ) {
		if ( 'material' == $arfinputstyle ) {
			if ( ARF_SIGNATURE_SLUG == $type ) {
				array_push( $arf_input_style_label_position, ARF_SIGNATURE_SLUG );
			}
		}
		return $arf_input_style_label_position;
	}
	function add_label_materialize_outside_editor( $field ) {
		if ( ARF_SIGNATURE_SLUG == $field['type'] ) { ?>
		<label class="arf_main_label" id="field_<?php echo esc_attr( $field['id'] ); ?>">
			<span class="arfeditorfieldopt_label arf_edit_in_place">
				<input type="text" class="arf_edit_in_place_input inplace_field" data-ajax="false" data-field-opt-change="true" data-field-opt-key='name' value="<?php echo esc_attr( $field['name'] ); ?>" data-field-id="<?php echo esc_attr( $field['id'] ); ?>" />
			</span>
			<span id="require_field_<?php echo esc_attr( $field['id'] ); ?>">
				<a href="javascript:arfmakerequiredfieldfunction('<?php echo esc_html( $field['id'] ); ?>',<?php echo $field_required = ( '0' == $field['required'] ) ? '0' : '1'; ?>,'1')" class="arfaction_icon arfhelptip arffieldrequiredicon alignleft arfcheckrequiredfield<?php echo esc_html( $field_required ); ?>" id="req_field_<?php echo esc_attr( $field['id'] ); ?>" title="<?php echo esc_html__( 'Click to mark as', 'ARForms-Digital-Signature' ) . ( '0' == $field['required'] ? ' ' : ' not ' ) . esc_html__( 'compulsory field.', 'ARForms-Digital-Signature' ); ?>"></a>
			</span>
		</label>
			<?php
		} }

	function arf_get_field_class( $class ) {
		global $arf_field_class_name;
		if ( $this->check_arforms_support() ) {
			return $class;
		}
		return array_merge( $class, $arf_field_class_name );
	}

	function arf_add_signature_filed_v3( $id = '', $is_ref_form = '', $values = '' ) {

		if ( $this->check_arforms_support() ) {
			return;
		}
		global $arf_field_class_name, $arf_new_field_data, $arf_field_image_path;

		foreach ( $arf_new_field_data as $field_key => $field_type ) {
			?>
			<li class="arf_form_element_item frmbutton frm_t<?php echo esc_html( $field_key ); ?>" id="<?php echo esc_attr( $field_key ); ?>" data-type="<?php echo esc_attr( $field_key ); ?>" data-field-id="<?php echo esc_attr( $id ); ?>">
				<div class="arf_form_element_item_inner_container">
					<div class="arf_form_element_item_icon">
						<svg viewBox="0 0 30 30"><g id="arfdigitaldownload"><path fill="#4E5462" d="M20.178,11.206l4.407,0.879l-1.989,9.979c-0.186,0.938-1.594,3.99-2.73,4.749l-0.17,0.857c-0.043,0.217-0.218,0.376-0.424,0.408c-0.059,0.01-0.119,0.008-0.181-0.003c-0.278-0.057-0.459-0.326-0.403-0.606l0.16-0.81c-0.81-1.111-0.811-4.691-0.654-5.476L20.178,11.206z M24.647,0.277c-1.211-0.24-2.417,0.837-2.637,1.944l-1.441,7.232l4.407,0.876l1.437-7.23C26.595,2.194,25.862,0.519,24.647,0.277z M18.051,11.089c0.124,0.024,0.247,0.026,0.363,0.009c0.413-0.064,0.762-0.379,0.85-0.813l1.531-7.559c0.113-0.557-0.248-1.1-0.804-1.212c-0.562-0.109-1.101,0.247-1.214,0.805l-1.531,7.559C17.133,10.434,17.495,10.977,18.051,11.089z M16.446,25.152c-0.313-0.217-0.705-0.485-1.295-0.645c-0.485-0.13-0.93-0.071-1.254-0.028c-0.033,0.006-0.072,0.011-0.111,0.015c-0.289-0.521-0.753-0.805-1.39-0.845c-0.841-0.046-1.431,0.44-1.835,0.807c0.004-0.232,0.028-0.513,0.047-0.716c0.057-0.651,0.115-1.324-0.098-1.914c-0.17-0.465-0.631-0.758-1.125-0.717c-1.271,0.127-2.193,0.966-3.005,1.721c-0.853,0.793-1.403,1.272-2.044,1.149c-0.783-0.147-1.172-2.144-1.096-3.987c0.067-1.661,0.192-4.753,1.5-6.353c0.44-0.539,1.105-1.151,1.752-1.006c0.761,0.17,1.538,1.257,1.891,2.643c0.147,0.585,0.734,0.939,1.329,0.791c0.587-0.147,0.94-0.744,0.791-1.33c-0.573-2.258-1.928-3.882-3.535-4.241c-0.963-0.21-2.419-0.08-3.923,1.758c-1.635,2.001-1.887,4.998-1.993,7.649c-0.054,1.324-0.009,5.684,2.876,6.229c1.758,0.333,2.967-0.795,3.939-1.697c0.178-0.166,0.353-0.328,0.524-0.476c-0.058,0.813-0.039,1.668,0.545,2.302c0.872,0.95,1.888,0.91,3.019-0.117c0.044-0.039,0.095-0.084,0.142-0.13c0.155,0.227,0.403,0.463,0.808,0.589c0.489,0.151,0.945,0.091,1.277,0.048c0.142-0.019,0.336-0.044,0.398-0.028c0.217,0.058,0.354,0.15,0.623,0.333c0.201,0.139,0.412,0.281,0.674,0.423c0.218,0.115,0.456,0.149,0.681,0.113	c0.329-0.051,0.632-0.252,0.798-0.568c0.283-0.533,0.08-1.195-0.453-1.479C16.726,25.348,16.584,25.247,16.446,25.152z"/></g>
					</div>
					<div class="arf_form_element_item_text"><?php echo esc_html( $field_type ); ?></div>
				</div>
			</li>
		<?php } ?>
		<?php
	}

	function arf_add_signature_filed( $id = '', $is_ref_form = '', $values = '' ) {

		if ( $this->check_arforms_support() ) {
			return;
		}
		global $arf_field_class_name, $arf_new_field_data, $arf_field_image_path;

		if ( is_rtl() ) {
			$floating_style = 'arf_float_right';
		} else {
			$floating_style = 'arf_float_left';
		}

		foreach ( $arf_new_field_data as $field_key  => $field_type ) {
			?>
			<li class="frmbutton arf_signature_icon_wrapper arf_signature_field_icon_wrapper <?php echo esc_attr( $floating_style ); ?> frm_t<?php echo esc_attr( $field_key ); ?> arffield_odd" id="<?php echo esc_attr( $field_key ); ?>" >
			 <div class="form-field">
				<div class="icon-bg_<?php echo esc_html( $arf_field_class_name[ $field_key ] ); ?>"><img src="<?php echo esc_url( $arf_field_image_path[ $field_key ] ); ?>" /></div>
				<div class="arrow_<?php echo esc_html( $arf_field_class_name[ $field_key ] ); ?>"></div>
				<div class="element-name"><?php echo apply_filters( 'arfaddnewfieldlinks', $field_type, $id, $field_key ); ?></div>
			  </div>
			</li>
			<?php } ?>
		<?php
	}

	function arf_new_field_json_filter_outside( $field_json ) {
		$field_data               = file_get_contents( ARF_DS_CORE . '/arf_signature_field_data.json' );
		$field_data_array         = json_decode( $field_data, true );
		$field_data_obj_signature = $field_data_array['field_data']['signature'];

		if ( $this->check_arforms_support() ) {
			return $field_json;
		}
		$field_json['field_data']['signature'] = $field_data_obj_signature;
		return $field_json;
	}

	function arf_new_field_array_filter_outside( $field_options_new, $field_icons, $filed_data_array, $positioned_field_icons ) {
		global $arfieldcontroller;

		$field_data               = file_get_contents( ARF_DS_CORE . '/arf_signature_field_data.json' );
		$field_data_array         = json_decode( $field_data );
		$arf_editor_unique_id     = 5656;
		$field_data_obj_signature = $field_data_array->field_data->signature;
		$field_opt_arr            = array(
			ARF_SIGNATURE_SLUG => array(
				'labelname'        => 1,
				'requiredmsg'      => 2,
				'fielddescription' => 3,
				'image_height'     => 4,
				'image_width'      => 5,
			),
		);
		$field_order_signature    = isset( $field_opt_arr[ ARF_SIGNATURE_SLUG ] ) ? $field_opt_arr[ ARF_SIGNATURE_SLUG ] : '';

		$singature_html = array(
			'signature' => "<div  class='arf_inner_wrapper_sortable arfmainformfield edit_form_item arffieldbox ui-state-default 1  arf1columns single_column_wrapper' data-id='arf_editor_main_row_{arf_editor_index_row}'><div class='arf_multiiconbox'><div class='arf_field_option_multicolumn' id='arf_multicolumn_wrapper'><input type='hidden' name='multicolumn' />{$field_icons['multicolumn_one']} {$field_icons['multicolumn_two']} {$field_icons['multicolumn_three']} {$field_icons['multicolumn_four']} {$field_icons['multicolumn_five']} {$field_icons['multicolumn_six']}</div>{$field_icons['multicolumn_expand_icon']}</div><div class='sortable_inner_wrapper edit_field_type_signature' id='arfmainfieldid_{arf_field_id}' inner_class='arf_1col'><div id='arf_field_{arf_field_id}' class='arfformfield control-group arfmainformfield top_container arf_field_{arf_field_id}'><div class='fieldname-row arf_dig_display_block' ><div class='fieldname'><label class='arf_main_label' id='field_{arf_field_id}'><span class='arfeditorfieldopt_label arf_edit_in_place'><input type='text' class='arf_edit_in_place_input inplace_field' data-ajax='false' data-field-opt-change='true' data-field-opt-key='name' value='Signature' data-field-id='{arf_field_id}' /></span><span id='require_field_{arf_field_id}'><a href='javascript:arfmakerequiredfieldfunction({arf_field_id},0,1)' class='arfaction_icon arfhelptip arffieldrequiredicon alignleft arfcheckrequiredfield0' id='req_field_{arf_field_id}' title='" . esc_html__( 'Click to mark as not compulsory field', 'ARForms-Digital-Signature' ) . "'></a></span></label></div></div><div class='arf_fieldiconbox' data-field_id='{arf_field_id}' data-field_id='{arf_field_id}'>" . $positioned_field_icons[ ARF_SIGNATURE_SLUG ] . "</div><div class='controls'><img class='arf_signature_edior_img arf_signature_editor_img_default' src='" . ARF_DIG_SIG_URL . "/images/signature-pad.png' /><input id='field_{arf_unique_key}_" . $arf_editor_unique_id . "' name='item_meta[{arf_field_id}]' type='hidden' class='arf_signature_output arf_dig_sig_float_left'><div class='arf_field_description' id='field_description_{arf_field_id}'></div><div class='help-block'></div></div><input type='hidden' name='arf_field_data_{arf_field_id}' id='arf_field_data_{arf_field_id}' value='" . htmlspecialchars( json_encode( $field_data_obj_signature ) ) . "' data-field_options='" . json_encode( $field_order_signature ) . "' /><div class='arf_field_option_model arf_field_option_model_cloned' data-field_id='{arf_field_id}'><div class='arf_field_option_model_header'>" . esc_html__( 'Field Options', 'ARForms-Digital-Signature' ) . "</div><div class='arf_field_option_model_container'><div class='arf_field_option_content_row'></div></div><div class='arf_field_option_model_footer'><button type='button' class='arf_field_option_close_button' onClick='arf_close_field_option_popup();'>" . esc_html__( 'Cancel', 'ARForms-Digital-Signature' ) . "</button><button type='button' class='arf_field_option_submit_button' data-field_id='{arf_field_id}'>" . esc_html__( 'OK', 'ARForms-Digital-Signature' ) . '</button></div></div></div></div>',
		);
		if ( $this->check_arforms_support() ) {
			return $field_options_new;
		}
		return array_merge( $field_options_new, $singature_html );
	}

	function arf_new_field_array_filter_outside_materialize( $field_options_new, $field_icons, $filed_data_array, $positioned_field_icons ) {
		global $arfieldcontroller;
		$field_data               = file_get_contents( ARF_DS_CORE . '/arf_signature_field_data.json' );
		$field_data_array         = json_decode( $field_data );
		$arf_editor_unique_id     = 5656;
		$field_data_obj_signature = $field_data_array->field_data->signature;
		$field_opt_arr            = array(
			ARF_SIGNATURE_SLUG => array(
				'labelname'        => 1,
				'requiredmsg'      => 2,
				'fielddescription' => 3,
				'image_height'     => 4,
				'image_width'      => 5,
			),
		);
		$field_order_signature    = isset( $field_opt_arr[ ARF_SIGNATURE_SLUG ] ) ? $field_opt_arr[ ARF_SIGNATURE_SLUG ] : '';
		$singature_html           = array(
			'signature' => "<div class='arf_inner_wrapper_sortable arfmainformfield edit_form_item arffieldbox ui-state-default 1  arf1columns single_column_wrapper' data-id='arf_editor_main_row_{arf_editor_index_row}'><div class='arf_multiiconbox'><div class='arf_field_option_multicolumn' id='arf_multicolumn_wrapper'><input type='hidden' name='multicolumn' />{$field_icons['multicolumn_one']} {$field_icons['multicolumn_two']} {$field_icons['multicolumn_three']} {$field_icons['multicolumn_four']} {$field_icons['multicolumn_five']} {$field_icons['multicolumn_six']}</div>{$field_icons['multicolumn_expand_icon']}</div><div class='sortable_inner_wrapper edit_field_type_signature' inner_class='arf_1col' id='arfmainfieldid_{arf_field_id}'><div id='arf_field_{arf_field_id}' class='arfformfield control-group arfmainformfield top_container  arfformfield  arf_field_{arf_field_id}'><label class='arf_main_label' id='field_{arf_field_id}'><span class='arfeditorfieldopt_label arf_edit_in_place'><input type='text' class='arf_edit_in_place_input inplace_field' data-ajax='false' data-field-opt-change='true' data-field-opt-key='name' value='Signature' data-field-id='{arf_field_id}' /></span><span id='require_field_{arf_field_id}'><a href='javascript:arfmakerequiredfieldfunction({arf_field_id},0,1)' class='arfaction_icon arfhelptip arffieldrequiredicon alignleft arfcheckrequiredfield0' id='req_field_{arf_field_id}' title='" . esc_html__( 'Click to mark as not compulsory field', 'ARForms-Digital-Signature' ) . "'></a></span></label><div class='fieldname-row arf_dig_display_block' ><div class='fieldname'></div></div><div class='arf_fieldiconbox' data-field_id='{arf_field_id}'>" . $positioned_field_icons[ ARF_SIGNATURE_SLUG ] . "</div><div class='controls input-field'><img class='arf_signature_edior_img arf_signature_editor_img_default' src='" . ARF_DIG_SIG_URL . "/images/signature-pad.png'><input id='field_{arf_unique_key}_" . $arf_editor_unique_id . "' name='item_meta[{arf_field_id}]' type='hidden' class='arf_signature_output arf_dig_sig_float_left'><div class='arf_field_description' id='field_description_{arf_field_id}'></div><div class='help-block'></div><input type='hidden' name='arf_field_data_{arf_field_id}' id='arf_field_data_{arf_field_id}' value='" . htmlspecialchars( json_encode( $field_data_obj_signature ) ) . "' data-field_options='" . json_encode( $field_order_signature ) . "' /><div class='arf_field_option_model arf_field_option_model_cloned' data-field_id='{arf_field_id}'><div class='arf_field_option_model_header'>" . esc_html__( 'Field Options', 'ARForms-Digital-Signature' ) . "</div><div class='arf_field_option_model_container'><div class='arf_field_option_content_row'></div></div><div class='arf_field_option_model_footer'><button type='button' class='arf_field_option_close_button' onClick='arf_close_field_option_popup();'>" . esc_html__( 'Cancel', 'ARForms-Digital-Signature' ) . "</button><button type='button' class='arf_field_option_submit_button' data-field_id='{arf_field_id}'>" . esc_html__( 'OK', 'ARForms-Digital-Signature' ) . '</button></div></div></div></div></div>',
		);
		if ( $this->check_arforms_support() ) {
			return $field_options_new;
		}
		return array_merge( $field_options_new, $singature_html );
	}
	function add_availablefieldsbasicoptions( $basic_option ) {

		$signature_filed_option = array(
			ARF_SIGNATURE_SLUG => array(
				'labelname'        => 1,
				'requiredmsg'      => 2,
				'fielddescription' => 3,
				'image_height'     => 4,
				'image_width'      => 5,
			),
		);

		if ( $this->check_arforms_support() ) {
			return $basic_option;
		}
		return array_merge( $basic_option, $signature_filed_option );
	}

	public static function install() {
		$arf_dig_sig_version = get_option( 'arf_dig_sig_version' );
		if ( ! isset( $arf_dig_sig_version ) || '' == $arf_dig_sig_version ) {
			global $arf_dig_sig_version;
			update_option( 'arf_dig_sig_version', $arf_dig_sig_version );
		}
	}

	public static function arf_signature_check_network_activation( $network_wide ) {
		if ( ! $network_wide ) {
			return;
		}

		deactivate_plugins( plugin_basename( __FILE__ ), true, true );

		header( 'Location: ' . network_admin_url( 'plugins.php?deactivate=true' ) );
		exit;
	}

	public static function uninstall() {
		global $wpdb;
		if ( is_multisite() ) {
			$blogs = $wpdb->get_results( "SELECT blog_id FROM {$wpdb->blogs}", ARRAY_A );
			if ( $blogs ) {
				foreach ( $blogs as $blog ) {
					switch_to_blog( $blog['blog_id'] );
					delete_option( 'arf_dig_sig_version' );
				}
				restore_current_blog();
			}
		} else {
			delete_option( 'arf_dig_sig_version' );
		}
	}

	function arf_dig_sig_getapiurl() {
		$api_url = 'https://www.arpluginshop.com/';
		return $api_url;
	}

	function arf_dig_sig_get_remote_post_params( $plugin_info = '' ) {
		global $wpdb, $arfversion;

		$action = '';
		$action = $plugin_info;

		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		$plugin_list = get_plugins();
		$site_url    = home_url();
		$plugins     = array();

		$active_plugins = get_option( 'active_plugins' );

		foreach ( $plugin_list as $key => $plugin ) {
			$is_active = in_array( $key, $active_plugins );

			if ( strpos( strtolower( $plugin['Title'] ), 'arformsignature' ) !== false ) {
				$name      = substr( $key, 0, strpos( $key, '/' ) );
				$plugins[] = array(
					'name'      => $name,
					'version'   => $plugin['Version'],
					'is_active' => $is_active,
				);
			}
		}
		$plugins = json_encode( $plugins );

		$theme            = wp_get_theme();
		$theme_name       = $theme->get( 'Name' );
		$theme_uri        = $theme->get( 'ThemeURI' );
		$theme_version    = $theme->get( 'Version' );
		$theme_author     = $theme->get( 'Author' );
		$theme_author_uri = $theme->get( 'AuthorURI' );

		$im        = is_multisite();
		$sortorder = get_option( 'arfSortOrder' );

		$post = array(
			'wp'        => get_bloginfo( 'version' ),
			'php'       => phpversion(),
			'mysql'     => $wpdb->db_version(),
			'plugins'   => $plugins,
			'tn'        => $theme_name,
			'tu'        => $theme_uri,
			'tv'        => $theme_version,
			'ta'        => $theme_author,
			'tau'       => $theme_author_uri,
			'im'        => $im,
			'sortorder' => $sortorder,
		);

		return $post;
	}

	function arfsignature_backup() {
		$arf_dig_sig_version = get_option( 'arf_dig_sig_version' );
		update_option( 'old_arf_dig_sig_version', $arf_dig_sig_version );
	}

	function upgrade_data() {

		global $newarf_dig_sig_version;

		if ( ! isset( $newarf_dig_sig_version ) || '' == $newarf_dig_sig_version ) {
			$newarf_dig_sig_version = get_option( 'arf_dig_sig_version' );
		}

		if ( version_compare( $newarf_dig_sig_version, '2.0', '<' ) ) {
			$path = ARF_DIG_SIG_DIR . '/core/upgrade_signature_latest_data.php';
			if ( file_exists( $path ) ) {
				include $path;
			}
		}
	}

	function arf_set_js() {
		global $arf_dig_sig_version,$arf_dig_sig_asset_version;
		if ( $this->check_arforms_support() ) {
			return;
		}

		if ( isset( $_REQUEST['page'] ) && '' != $_REQUEST['page'] && ( 'ARForms' == $_REQUEST['page'] && isset( $_REQUEST['arfaction'] ) && ( 'edit' == $_REQUEST['arfaction'] || 'new' == $_REQUEST['arfaction'] || 'duplicate' == $_REQUEST['arfaction'] || 'update' == $_REQUEST['arfaction'] ) ) ) {
			wp_register_script( 'arfdigsig-js', ARF_DIG_SIG_URL . '/js/arf_dig_sig.js', array( 'jquery' ), $arf_dig_sig_asset_version );
			wp_register_script( 'jquery-json2', ARF_DIG_SIG_URL . '/js/json2.min.js', array(), $arf_dig_sig_asset_version );
			wp_register_script( 'signature_pad_umd_lib', ARF_DIG_SIG_URL . '/js/signature_pad.umd.js', array(), $arf_dig_sig_asset_version );
			wp_enqueue_script( 'arfdigsig-js' );
			wp_enqueue_script( 'jquery-json2' );
			wp_enqueue_script( 'signature_pad_umd_lib' );
		}
		if ( isset( $_REQUEST['page'] ) && '' != $_REQUEST['page'] && ( 'ARForms-entries' == $_REQUEST['page'] && isset( $_REQUEST['form'] ) ) ) {
			wp_register_script( 'arfdigsigentries-js', ARF_DIG_SIG_URL . '/js/arf_dig_sig_entries.js', array( 'jquery' ), $arf_dig_sig_asset_version );
			wp_enqueue_script( 'arfdigsigentries-js' );
		}
	}

	function arf_set_js_outside_preview_header() {
		global $arf_dig_sig_version,$arf_dig_sig_asset_version;
		if ( $this->check_arforms_support() ) {
			return false;
		}
		wp_register_script( 'arfdigsig-js', ARF_DIG_SIG_URL . '/js/arf_dig_sig.js', array( 'jquery' ), $arf_dig_sig_asset_version );
		wp_register_script( 'jquery-json2', ARF_DIG_SIG_URL . '/js/json2.min.js', array(), $arf_dig_sig_asset_version );
		wp_register_script( 'signature_pad_umd_lib', ARF_DIG_SIG_URL . '/js/signature_pad.umd.js', array(), $arf_dig_sig_asset_version );
		wp_print_scripts( 'arfdigsig-js' );
		wp_print_scripts( 'jquery-json2' );
		wp_print_scripts( 'signature_pad_umd_lib' );
		wp_register_style( 'signaturepad-css', ARF_DIG_SIG_URL . '/css/arf_dig_sig_frontend.css', array(), $arf_dig_sig_asset_version );
		wp_register_style( 'arf-dig-sig-admin-css', ARF_DIG_SIG_URL . '/css/arf_dig_sig_admin.css', array(), $arf_dig_sig_asset_version );
		wp_print_styles( 'signaturepad-css' );
		wp_print_styles( 'arf-dig-sig-admin-css' );
	}

	function arf_front_footer_js( $loaded_field ) {

		if ( $this->check_arforms_support() ) {
			return;
		}

		global $arf_dig_sig_version,$arf_dig_sig_asset_version;
		wp_register_script( 'arfdigsig-js', ARF_DIG_SIG_URL . '/js/arf_dig_sig.js', array( 'jquery' ), $arf_dig_sig_asset_version );
		wp_register_script( 'jquery-json2', ARF_DIG_SIG_URL . '/js/json2.min.js', array(), $arf_dig_sig_asset_version );
		wp_register_script( 'signature_pad_umd_lib', ARF_DIG_SIG_URL . '/js/signature_pad.umd.js', array(), $arf_dig_sig_asset_version );
		wp_register_style( 'signaturepad-css', ARF_DIG_SIG_URL . '/css/arf_dig_sig_frontend.css', array(), $arf_dig_sig_asset_version );
		global $arfforms_loaded;
		wp_enqueue_script( 'arfdigsig-js' );
		wp_enqueue_script( 'jquery-json2' );
		wp_enqueue_script( 'signature_pad_umd_lib' );
		wp_enqueue_style( 'signaturepad-css' );

		$signature_internal_css = '.arf_dig_sig_current .arf_dig_sig_pad{ cursor: url("' . ARF_DIG_SIG_URL . '/images/pen.cur"), crosshair;}';

		wp_add_inline_style( 'signaturepad-css', $signature_internal_css );
	}

	function arf_set_css( $hook ) {
		if ( $this->check_arforms_support() ) {
			return;
		}
		global $arf_dig_sig_version,$arf_dig_sig_asset_version;
		wp_register_style( 'arf-dig-sig-admin-css', ARF_DIG_SIG_URL . '/css/arf_dig_sig_admin.css', array(), $arf_dig_sig_asset_version );

		if ( isset( $_REQUEST['page'] ) && '' != $_REQUEST['page'] && ( 'ARForms' == $_REQUEST['page'] && isset( $_REQUEST['arfaction'] ) && ( 'edit' == $_REQUEST['arfaction'] || 'new' == $_REQUEST['arfaction'] || 'duplicate' == $_REQUEST['arfaction'] || 'update' == $_REQUEST['arfaction'] ) ) ) {
			wp_register_style( 'signaturepad-css', ARF_DIG_SIG_URL . '/css/arf_dig_sig_frontend.css', array(), $arf_dig_sig_asset_version );
			wp_enqueue_style( 'signaturepad-css' );

			wp_enqueue_style( 'arf-dig-sig-admin-css' );
		} elseif ( isset( $_REQUEST['page'] ) && 'ARForms-entries' == $_REQUEST['page'] ) {
			wp_enqueue_style( 'arf-dig-sig-admin-css' );
		} elseif ( isset( $hook ) && 'plugins.php' == $hook ) {
			global $wp_version;

			if ( version_compare( $wp_version, '4.5.0', '<' ) ) {
				wp_print_styles( 'arf-dig-sig-admin-css' );
				global $arf_dig_sig_asset_version;
				deactivate_plugins( plugin_basename( __FILE__ ), true, false );
				$redirect_url = network_admin_url( 'plugins.php?deactivate=true' );
				wp_die( '<div class="arf_dig_sig_wp_notice"><p class="arf_dig_sig_wp_notice_text" >Please meet the minimum requirement of WordPress version 4.5 to activate ARForms - Signature Add-on<p class="arf_dig_sig_wp_notice_continue">Please <a href="javascript:void(0)" onclick="window.location.href=\'' . $redirect_url . '\'">Click Here</a> to continue.</p></div>' );
			}
		}
	}

	function add_field_to_editor( $field ) {
		global $arfajaxurl;
		$field_name = 'item_meta[' . $field['id'] . ']';
		require ARF_DS_CORE . '/displayfield_in_editor.php';
	}

	function add_field_to_frontend_v3( $return_string, $form, $field_name, $arf_data_uniq_id, $field, $field_tootip, $field_description, $res_data, $inputStyle, $arf_main_label ) {

		global $style_settings, $arfsettings, $arfeditingentry, $arffield;
		$entry_id = $arfeditingentry;

		if ( $inputStyle == 'material' ) {
			require ARF_DS_CORE . '/displayfield_in_frontend_materialize_3.0.php';
		} else {
			require ARF_DS_CORE . '/displayfield_in_frontend_3.0.php';
		}

		return $return_string;
	}

	function add_field_to_frontend_materialize_v3( $return_string, $form, $field_name, $arf_data_uniq_id, $field, $field_tootip, $field_description, $res_data, $inputStyle, $arf_main_label ) {

		global $style_settings, $arfsettings, $arfeditingentry, $arffield;
		$entry_id = $arfeditingentry;

		require ARF_DS_CORE . '/displayfield_in_frontend_materialize_3.0.php';

		return $return_string;
	}
	function add_field_to_frontend( $field, $field_name ) {

		global $style_settings, $arfsettings, $arfeditingentry, $arffield,$is_version_compatible;
		$entry_id = $arfeditingentry;

		require ARF_DS_CORE . '/displayfield_in_frontend.php';

	}

	function arf_display_signature_value( $value = '', $field = '', $atts = '' ) {
		global $arf_digital_signature;

		if ( $atts['type'] == ARF_SIGNATURE_SLUG ) {
			if ( ! empty( $value ) ) {
				$image_info = pathinfo( $value );
				if ( isset( $atts['is_remove_icon_with_edit'] ) && $atts['is_remove_icon_with_edit'] == 1 ) {
					$value = '<span class="arf_deletable_entry_icon_wrapper arf_signature_inner_' . $image_info['filename'] . '"><a data-id="' . $field->id . '" data-entry-id="' . $atts['entry_id'] . '" data-file="' . $image_info['filename'] . '" data-file-url="' . $value . '" class="arf_signaturefile_remove arf_dig_cursor_pointer"><svg height="28" width="28"><g><path fill="#FF0000" d="M18.435,4.857L18.413,19.87L3.398,19.88L3.394,4.857H1.489V2.929  h1.601h3.394V0.85h8.921v2.079h3.336h1.601l0,0v1.928H18.435z M15.231,4.857H6.597H5.425l0.012,13.018h10.945l0.005-13.018H15.231z   M11.4,6.845h2.029v9.065H11.4V6.845z M8.399,6.845h2.03v9.065h-2.03V6.845z"></path></g></svg></a></span><span class="arf_not_editable_values_container arf_signature_inner_' . $image_info['filename'] . '">' . $arf_digital_signature->arf_get_signature_image( $value, true ) . '</span>';
				} else {
					$value = $arf_digital_signature->arf_get_signature_image( $value, true );
				}
			}
		}
		return $value;
	}

	function arf_display_signature_value_in_pdf( $field ) {
		global $arf_digital_signature;
		if ( $field->type == ARF_SIGNATURE_SLUG ) {
			$height = $field->field_options->image_height;

			$width = $field->field_options->image_width;

			if ( $width > 700 ) {
				$temp   = ( $height / $width );
				$width  = 700;
				$height = 700 * $temp;
			}

			global $display_in_pdf;
			$display_in_pdf = 1;
			$field->entry_value = $arf_digital_signature->arf_get_signature_image( $field->entry_value, '', $width, $height, 1 );
		}
		return $field;
	}

	function arf_display_signature_value_in_mail( $value, $entry_id, $form_id ) {
		global $arf_digital_signature;

		if ( $value->field_type == ARF_SIGNATURE_SLUG ) {
			$height = isset( $value->field_options['image_height'] ) ? $value->field_options['image_height'] : 150;

			$width = isset( $value->field_options['image_width'] ) ? $value->field_options['image_width'] : 300;

			if ( $width > 700 ) {
				$temp   = ( $height / $width );
				$width  = 700;
				$height = 700 * $temp;
			}
			$value->entry_value = $arf_digital_signature->arf_get_signature_image( $value->entry_value, true, $width, $height );
		}
		return $value;
	}

	function arf_get_signature_image( $sing_value, $link = '', $width = 100, $height = 100 ) {
		if ( empty( $sing_value ) ) {
			return false;
		}
		$wp_upload_url       = wp_upload_dir();
		$signature_image_url = $wp_upload_url['baseurl'] . '/arforms/signature/';
		$signature_image_dir = $wp_upload_url['basedir'] . '/arforms/signature/';
		if ( is_ssl() ) {
			$signature_image_url = str_replace( 'http://', 'https://', $wp_upload_url['baseurl'] . '/arforms/signature/' );
		} else {
			$signature_image_url = $wp_upload_url['baseurl'] . '/arforms/signature/';
		}

		if( preg_match( '/\<img(\s+)src=(\"|\')(.*?)(\"|\')(.*?)\>/', $sing_value, $img_value ) ){
			if( isset( $img_value[3] ) ){
				$sing_value = $img_value[3];
			}
		}

		$as__sing_value = explode( '/', $sing_value );
		$as_sing_name   = end( $as__sing_value );

		if ( file_exists( $signature_image_dir . $as_sing_name ) ) {
			global $display_in_pdf;
			if ( ! empty( $link ) ) {
				$image_info = pathinfo( $sing_value );

				if ( isset( $display_in_pdf ) && $display_in_pdf == 1 ) {
					return "<a class='arf_signature_inner_" . $image_info['filename'] . "' href='" . $sing_value . "' target='_blank'><img src='" . $signature_image_url . $as_sing_name . "' width='" . $width . "' height='" . $height . "' /></a>";
				} else {
					return "<a class='arf_signature_inner_" . $image_info['filename'] . "' href='" . $sing_value . "' target='_blank'><img src='" . $sing_value . "' width='" . $width . "' height='" . $height . "' /></a>";
				}
			} else {
				if ( isset( $display_in_pdf ) && $display_in_pdf == 1 ) {
					return "<img src='" . $signature_image_url . $as_sing_name . "' width='" . $width . "' height='" . $height . "' />";
				} else {
					return "<img src='" . $sing_value . "' width='" . $width . "' height='" . $height . "' />";
				}
			}
		} else {
			return false;
		}
	}

	function arf_generate_signature_image( $sing, $image_width = '400', $image_height = '250' ) {
		if ( $sing == '' ) {
			return false;
		}
		global $arfjsonimage, $arfds_memory_limit, $memory_limit,$wp_filesystem;

		if ( isset( $arfds_memory_limit ) && isset( $memory_limit ) && $arfds_memory_limit > $memory_limit ) {
			@ini_set( 'memory_limit', $arfds_memory_limit . 'M' );
		}

		$name_prefix = rand( 1, 9999999 );
		$filename    = $name_prefix . '.png';

		$img                 = str_replace( 'data:image/png;base64,', '', $sing );
		$img                 = base64_decode( $img );
		$wp_upload_dir       = wp_upload_dir();
		$signature_image_dir = $wp_upload_dir['basedir'] . '/arforms/signature/';

		if ( ! is_dir( $signature_image_dir ) ) {
			wp_mkdir_p( $signature_image_dir );
		}

		$path = $signature_image_dir . $filename;

		if ( file_exists( ABSPATH . 'wp-admin/includes/file.php' ) ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
			if ( false === ( $creds = request_filesystem_credentials( $path, '', false, false ) ) ) {
				return true;
			}
			if ( ! WP_Filesystem( $creds ) ) {
				request_filesystem_credentials( $path, $method, true, false );
				return true;
			}
			$wp_filesystem->put_contents( $path, $img, 0755 );
		}

		return $filename;
	}

	function arf_before_create_entry( $values ) {

		global $arf_digital_signature, $arffield, $arfieldhelper;
		$wp_upload_url       = wp_upload_dir();
		$signature_image_url = $wp_upload_url['baseurl'] . '/arforms/signature/';

		if ( is_ssl() ) {
			$signature_image_url = str_replace( 'http://', 'https://', $wp_upload_url['baseurl'] . '/arforms/signature/' );
		} else {
			$signature_image_url = $wp_upload_url['baseurl'] . '/arforms/signature/';
		}

		if ( ! empty( $values['item_meta'] ) ) {
			foreach ( $values['item_meta'] as $field_id => $field_value ) {
				$field_type = $arfieldhelper->get_field_type( $field_id );
				if ( $field_type == 'signature' ) {
					$field_data = $arffield->getOne( $field_id );

					$field_options = $field_data->field_options;

					$imgnmae = $arf_digital_signature->arf_generate_signature_image( $field_value, $field_options['image_width'], $field_options['image_height'] );
					if ( $imgnmae ) {
						$values['item_meta'][ $field_id ] = $signature_image_url . $imgnmae;
					}
				}
			}
		}

		return $values;
	}

	function arf_delete_signature_image( $id ) {
		if ( empty( $id ) ) {
			return false;
		}
		global $wpdb, $db_record;

		$entry = $db_record->getOne( $id );

		$field_data = $wpdb->get_results( $wpdb->prepare( 'SELECT id,field_key,name, type, form_id FROM ' . $wpdb->prefix . 'arf_fields WHERE form_id = %d', $entry->form_id ) );

		$wp_upload_dir       = wp_upload_dir();
		$signature_image_dir = $wp_upload_dir['basedir'] . '/arforms/signature/';

		if ( ! empty( $field_data ) ) {
			foreach ( $field_data as $key => $field ) {
				$sing_image = '';
				if ( ARF_SIGNATURE_SLUG == $field->type ) {
					$sing_image = $wpdb->get_var( $wpdb->prepare( 'SELECT entry_value FROM ' . $wpdb->prefix . 'arf_entry_values WHERE field_id=%s and entry_id=%s', $field->id, $entry->id ) );
					if ( ! empty( $sing_image ) ) {
						if ( file_exists( $signature_image_dir . $sing_image ) ) {
							unlink( $signature_image_dir . $sing_image );
						}
					}
				}
			}
		}
		return $id;
	}

	function arf_createfield( $field_data ) {
		if ( ARF_SIGNATURE_SLUG == $field_data['type'] ) {

			$field_data['field_options']['image_width']  = '300';
			$field_data['field_options']['image_height'] = '150';
			$field_data['name']                          = esc_html__( 'Signature', 'ARForms-Digital-Signature' );
		}
		return $field_data;
	}

	function arf_create_css( $new_values, $use_saved = '', $id = '' ) {
		global $arf_digital_signature;
		if ( $arf_digital_signature->check_arforms_support() ) {
			return;
		}
		if ( isset( $use_saved ) && $use_saved ) {

			global $arsettingcontroller;

			extract( (array) $new_values );

			$form_id = $id;

			$form_border_shadow_color = $new_values['arfmainformbordershadowcolorsetting'];

			$field_text_color_pg_break = $new_values['text_color_pg_break'];

			$field_bg_color_pg_break = $new_values['bg_color_pg_break'];

			$field_bg_inactive_color_pg_break = $new_values['bg_inavtive_color_pg_break'];

			$field_border_style = $new_values['arffieldborderstylesetting'];

			$border_radius = ( '' == $new_values['border_radius'] ) ? '0px' : $new_values['border_radius'] . 'px';

			$field_border_width = ( '' == $new_values['arffieldborderwidthsetting'] ) ? '0px' : $new_values['arffieldborderwidthsetting'] . 'px';

			$field_border_width_select = ( '' == $new_values['arffieldborderwidthsetting'] ) ? '0' : $new_values['arffieldborderwidthsetting'];

			$border_width_error = $field_border_width;

			$border_color_error = $new_values['arferrorbordercolorsetting'];

			$field_height = isset( $field_height ) ? $field_height . 'px' : 'auto';
		} elseif ( isset( $_REQUEST['arfmfws'] ) ) {

			$form_id = $_REQUEST['arfmf'];

			$form_width_unit = $_REQUEST['arffu'];

			$field_width_unit = $_REQUEST['arffiu'];

			$width_unit = $_REQUEST['arfmwu'];

			$bg_color = $_REQUEST['arffmbc'];

			$width = $_REQUEST['arfmws'] . $width_unit;

			$field_width_unit = $_REQUEST['arffiu'];

			$border_color = $_REQUEST['arffmboc'];

			$field_border_width = ( '' == $_REQUEST['arffbws'] ) ? '0px' : $_REQUEST['arffbws'] . 'px';

			$field_border_width_select = ( '' == $_REQUEST['arffbws'] ) ? '0' : $_REQUEST['arffbws'];

			$field_border_style = $_REQUEST['arffbss'];

			$border_width_error = $field_border_width;

			$border_style_error = isset( $_REQUEST['arfbess'] ) ? $_REQUEST['arfbess'] : '';

			$border_radius = ( ! isset( $_REQUEST['arfmbs'] ) && '' == $_REQUEST['arfmbs'] ) ? '0px' : $_REQUEST['arfmbs'] . 'px';

			$field_height = ( empty( $_REQUEST['arfmfhs'] ) ) ? 'auto' : $_REQUEST['arfmfhs'] . 'px';

			$req_arfestc         = isset( $_REQUEST['arfestc'] ) ? $_REQUEST['arfestc'] : '';
			$arferrorstylecolor1 = explode( '|', $req_arfestc );

			$arferrorstylecolor     = isset( $arferrorstylecolor1[0] ) ? $arferrorstylecolor1[0] : '';
			$arferrorstylecolorfont = isset( $arferrorstylecolor1[1] ) ? $arferrorstylecolor1[1] : '';

			$arfvalidationerrorstyle = $_REQUEST['arfest'];

			$border_color_error = $_REQUEST['arfboecs'];
		}

		$hex = str_replace( '#', '', $bg_color );
		if ( strlen( $hex ) == 3 ) {
			$r = hexdec( substr( $hex, 0, 1 ) . substr( $hex, 0, 1 ) );
			$g = hexdec( substr( $hex, 1, 1 ) . substr( $hex, 1, 1 ) );
			$b = hexdec( substr( $hex, 2, 1 ) . substr( $hex, 2, 1 ) );
		} else {
			$r = hexdec( substr( $hex, 0, 2 ) );
			$g = hexdec( substr( $hex, 2, 2 ) );
			$b = hexdec( substr( $hex, 4, 2 ) );
		}
		$rgb = array( $r, $g, $b );

		$rgb_bg_color = implode( ',', $rgb );
		?>
		.ar_main_div_<?php echo esc_html( $form_id ); ?> .arf_dig_sig_sigWrapper{border-color:<?php echo esc_html( $border_color ); ?> !important;border-width:<?php echo esc_html( $field_border_width ); ?> !important;border-style:<?php echo esc_html( $field_border_style ); ?>;-moz-border-radius:<?php echo esc_html( $border_radius ); ?> !important;-webkit-border-radius:<?php echo esc_html( $border_radius ); ?> !important;border-radius:<?php echo esc_html( $border_radius ); ?> !important; background-color:<?php echo esc_html( $bg_color ); ?> !important;  }

		.ar_main_div_<?php echo esc_html( $form_id ); ?>  .arf_dig_sig_sigWrapper .arf_dig_sig_pad,{
		-moz-border-radius:<?php echo esc_html( $border_radius ); ?> !important;
		-webkit-border-radius:<?php echo esc_html( $border_radius ); ?> !important;
		border-radius:<?php echo esc_html( $border_radius ); ?> !important;
		background-color: rgb(<?php echo esc_html( $rgb_bg_color ); ?>) !important;

		-webkit-box-sizing:border-box;
		-moz-box-sizing:border-box;
		box-sizing:border-box;
		}

		.ar_main_div_<?php echo esc_html( $form_id ); ?> .control-group.arf_error .arf_dig_sig_sigWrapper{
		border-color: <?php echo esc_html( $border_color_error ); ?> !important;
		-webkit-box-shadow:none;
		-moz-box-shadow:none;
		box-shadow:none;
		}

		<?php
	}

	function arf_dig_sig_admin_notices() {
		global $wp_version;
		if ( version_compare( $wp_version, '4.5.0', '<' ) ) {
			deactivate_plugins( plugin_basename( __FILE__ ), true, false );

			echo "<div class='error arf_error'><p>Please meet the minimum requirement of WordPress version 4.5 to activate ARForms - Signature Add-on</p></div>";
		}

		if ( ! $this->is_arforms_support() ) {
			echo "<div class='updated'><p>" . esc_html__( 'Signature add-on for ARForms requires ARForms plugin installed and active.', 'ARForms-Digital-Signature' ) . '</p></div>';
		} elseif ( ! version_compare( $this->get_arforms_version(), '2.7', '>=' ) ) {
			echo "<div class='updated'><p>" . esc_html__( 'Signature add-on for ARForms requires ARForms installed with version 2.7 or higher.', 'ARForms-Digital-Signature' ) . '</p></div>';
		}
	}

	function is_arforms_support() {
		if ( file_exists( ABSPATH . 'wp-admin/includes/plugin.php' ) ) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		return is_plugin_active( 'arforms/arforms.php' );
	}

	function get_arforms_version() {

		$arf_db_version = get_option( 'arf_db_version' );

		return ( isset( $arf_db_version ) ) ? $arf_db_version : 0;
	}

	function check_arforms_support() {

		if ( ! version_compare( $this->get_arforms_version(), '2.7', '>=' ) ) {
			return true;
		} else {
			return false;
		}
	}

	function arf_add_signature_style( $field, $custom_css_block, $arf_data_uniq_id ) {

		if ( $field['type'] == 'signature' && $custom_css_block == 'css_input_element' ) {
			?>
			.ar_main_div_<?php echo esc_html( $field['form_id'] ); ?>  #arf_field_<?php echo esc_html( $field['id'] ); ?>_<?php echo esc_html( $arf_data_uniq_id ); ?>_container .controls .arf_dig_sig_sigWrapper .arf_dig_sig_pad{
			<?php echo esc_html( $field['css_input_element'] ); ?>
			};
			<?php
		}
	}

	function is_arforms_version() {
		if ( version_compare( $this->get_arforms_version(), '3.0', '>=' ) ) {
			return true;
		} else {
			return false;
		}
	}

	function arf_dig_sig_field_css_v3( $new_values, $use_saved, $form_id ) {
		global $wpdb,$MdlDb;
		
		if( isset($_REQUEST['arfaction']) && 'preview' == $_REQUEST['arfaction'] ){
			$options = get_option($_REQUEST['arf_opt_id']);
			$posted_data =  json_decode( stripslashes_deep( $options['posted_data'] ) ,true );

			$signature_fields = array();
			$i = 0;
			foreach( $posted_data as $k => $post_data ){
				if( preg_match('/arf_field_data_/',$k) && preg_match('/"type":"signature"/',$post_data) ){
					$field_id = str_replace('arf_field_data_','',$k);
					$signature_fields[$i] = new stdClass();
					$signature_fields[$i]->field_options = $post_data;
					$signature_fields[$i]->id = $field_id;
					$i++;
				}
			}
		} else {
			$signature_fields = $wpdb->get_results( $wpdb->prepare( 'SELECT field_options,id FROM `' . $MdlDb->fields . '` WHERE type = %s AND form_id = %d', 'signature', $form_id ) );
		}

		$form_type = '';
		if ( $new_values['arfinputstyle'] == 'material' ) {
			$form_type = ' .arf_materialize_form ';
		}

		foreach ( $signature_fields as $key => $signature ) {
			$field_opts = json_decode( $signature->field_options, true );

			if( '' == $field_opts['image_height'] ){
				$field_opts['image_height'] = 150;
			}
			$img_height = (((int)$field_opts['image_height'] ) + 50 );
			echo '.ar_main_div_' . $form_id . ' ' . $form_type . ' .arf_field_' . $signature->id . ' .controls{width:' . $field_opts['image_width'] . 'px;}';
			echo '.ar_main_div_' . $form_id . ' ' . $form_type . ' .arf_field_' . $signature->id . ' div.arf_dig_sig_pad{width:' . $field_opts['image_width'] . 'px;height:' . $img_height . 'px;}';
			echo '.ar_main_div_' . $form_id . ' ' . $form_type . ' .arf_field_' . $signature->id . ' input.arf_signature_output_' . $signature->id . '{ position:absolute; top:0; left:0; z-index: -1; }';
		}

	}

	function arf_dig_sig_field_css( $new_values, $use_saved, $form_id ) {
		global $wpdb,$MdlDb;
		$signature_fields = $wpdb->get_results( $wpdb->prepare( 'SELECT field_options,id,ref_field_id FROM `' . $MdlDb->fields . '` WHERE type = %s AND form_id = %d', 'signature', $form_id ) );

		foreach ( $signature_fields as $key => $signature ) {
			$field_opts = json_decode( $signature->field_options, true );

			if ( json_last_error() != JSON_ERROR_NONE ) {
				$field_opts = maybe_unserialize( $signature->field_options );
			}
			echo '.ar_main_div_' . $form_id . ' .arfformfield .arf_smoothed_' . $signature->id . '{width:' . $field_opts['image_width'] . 'px;height:' . $field_opts['image_height'] . 'px;}';
			echo '.ar_main_div_' . $form_id . ' .arf_field_' . $signature->id . '_container .controls{width:' . $field_opts['image_width'] . 'px;height:' . $field_opts['image_height'] . 'px;}';
			echo '.ar_main_div_' . $form_id . ' .arfformfield .arf_smoothed_' . $signature->ref_field_id . '{width:' . $field_opts['image_width'] . 'px;height:' . $field_opts['image_height'] . 'px;}';
			echo '.ar_main_div_' . $form_id . ' .arf_field_' . $signature->ref_field_id . '_container .controls{width:' . $field_opts['image_width'] . 'px;height:' . $field_opts['image_height'] . 'px;}';
		}
	}
}

global $arf_dig_sig_api_url, $arf_dig_sig_plugin_slug;

$arf_dig_sig_api_url     = $arf_digital_signature->arf_dig_sig_getapiurl();
$arf_dig_sig_plugin_slug = basename( dirname( __FILE__ ) );

add_filter( 'pre_set_site_transient_update_plugins', 'arf_dig_sig_check_for_plugin_update' );

function arf_dig_sig_check_for_plugin_update( $checked_data ) {
	global $arf_dig_sig_api_url, $arf_dig_sig_plugin_slug, $wp_version, $arf_digital_signature, $arf_dig_sig_version;

	if ( empty( $checked_data->checked ) ) {
		return $checked_data;
	}

	$args = array(
		'slug'            => $arf_dig_sig_plugin_slug,
		'version'         => $arf_dig_sig_version,
		'other_variables' => $arf_digital_signature->arf_dig_sig_get_remote_post_params(),
	);

	$request_string = array(
		'body'       => array(
			'action'  => 'basic_check',
			'request' => serialize( $args ),
			'api-key' => md5( home_url() ),
		),
		'user-agent' => 'ARFDIGSIG-WordPress/' . $wp_version . '; ' . home_url(),
	);

	$raw_response = wp_remote_post( $arf_dig_sig_api_url, $request_string );

	if ( ! is_wp_error( $raw_response ) && ( 200 == $raw_response['response']['code'] ) ) {
		$response = maybe_unserialize( $raw_response['body'] );
	}

	if ( isset( $response ) && ! empty( $response ) && isset( $response->token ) && '' != $response->token ) {
		update_option( 'arfdigsig_update_token', $response->token );
	}

	if ( isset( $response ) && is_object( $response ) && is_object( $checked_data ) && ! empty( $response ) ) {
		$checked_data->response[ $arf_dig_sig_plugin_slug . '/' . $arf_dig_sig_plugin_slug . '.php' ] = $response;
	}

	return $checked_data;
}

add_filter( 'plugins_api', 'arf_dig_sig_plugin_api_call', 10, 3 );

function arf_dig_sig_plugin_api_call( $def, $action, $args ) {
	global $arf_dig_sig_plugin_slug, $arf_dig_sig_api_url, $wp_version;

	if ( ! isset( $args->slug ) || ( $args->slug != $arf_dig_sig_plugin_slug ) ) {
		return false;
	}

	$plugin_info     = get_site_transient( 'update_plugins' );
	$current_version = $plugin_info->checked[ $arf_dig_sig_plugin_slug . '/' . $arf_dig_sig_plugin_slug . '.php' ];
	$args->version   = $current_version;

	$request_string = array(
		'body'       => array(
			'action'       => $action,
			'update_token' => get_site_option( 'arfdigsig_update_token' ),
			'request'      => serialize( $args ),
			'api-key'      => md5( home_url() ),
		),
		'user-agent' => 'ARFDIGSIG-WordPress/' . $wp_version . '; ' . home_url(),
	);

	$request = wp_remote_post( $arf_dig_sig_api_url, $request_string );

	if ( is_wp_error( $request ) ) {
		$res = new WP_Error( 'plugins_api_failed', 'An Unexpected HTTP Error occurred during the API request.</p> <p><a href="?" onclick="document.location.reload(); return false;">Try again</a>', $request->get_error_message() );
	} else {
		$res = maybe_unserialize( $request['body'] );

		if ( false === $res ) {
			$res = new WP_Error( 'plugins_api_failed', 'An unknown error occurred', $request['body'] );
		}
	}

	return $res;
}

?>